@@ -14,11 +14,28 @@
 
 # This script is executed inside post_test_hook function in devstack gate.
 
-cd /opt/stack/new/mistral/functionaltests
-sudo ./run_tests.sh
-RETVAL=$?
 
-# Copy tempest log files to be published among other logs upon job completion
-sudo cp /opt/stack/new/mistral/functionaltests/tempest.log /opt/stack/logs
+RETVAL=0
+
+sudo chmod -R a+rw /opt/stack/new/
+cd /opt/stack/new/
+
+echo "Repository: $ZUUL_PROJECT"
+
+#Run API tests only for mistral repository
+if [[ "$ZUUL_PROJECT" == "openstack/mistral" ]]; then
+    cd mistral/
+    echo "Run mistral API tests"
+    ./functionaltests/run_tests.sh
+    RETVAL=$?
+fi
+
+#Run client tests for both repositories: mistral and python-mistralclient
+if [[ RETVAL -eq 0 ]]; then
+    cd /opt/stack/new/python-mistralclient/
+    echo "Run mistralclient tests"
+    ./functionaltests/run_tests.sh
+    RETVAL=$?
+fi
 
 exit $RETVAL
