@@ -203,7 +203,9 @@ class DbTestCase(BaseTest):
 
     @classmethod
     def __heavy_init(cls):
-        """Make this method private to prevent extending this one.
+        """Method that runs heavy_init().
+
+        Make this method private to prevent extending this one.
         It runs heavy_init() only once.
 
         Note: setUpClass() can be used, but it magically is not invoked
@@ -215,7 +217,9 @@ class DbTestCase(BaseTest):
 
     @classmethod
     def heavy_init(cls):
-        """Runs a long initialization (runs once by class)
+        """Runs a long initialization.
+
+        This method runs long initialization  once by class
         and can be extended by child classes.
         """
         # If using sqlite, change to memory. The default is file based.
