@@ -20,6 +20,7 @@ import mock
 import six
 
 from tempest import clients
+from tempest.common import credentials_factory as creds
 from tempest import config
 from tempest import test as test
 from tempest_lib import auth
@@ -88,14 +89,19 @@ class MistralClientBase(rest_client.RestClient):
         return resp, json.loads(body)
 
     def wait_execution_success(self, ex_body, timeout=180, url='executions'):
+        return self.wait_execution(ex_body, timeout=timeout, url=url)
+
+    def wait_execution(self, ex_body, timeout=180, url='executions',
+                       target_state='SUCCESS'):
         start_time = time.time()
 
-        expected_states = ['SUCCESS', 'RUNNING']
+        expected_states = [target_state, 'RUNNING']
 
-        while ex_body['state'] != 'SUCCESS':
+        while ex_body['state'] != target_state:
             if time.time() - start_time > timeout:
-                msg = ("Execution exceeds timeout {0} to change state "
-                       "to SUCCESS. Execution: {1}".format(timeout, ex_body))
+                msg = ("Execution exceeds timeout {0} "
+                       "to change state to {1}. "
+                       "Execution: {2}".format(timeout, target_state, ex_body))
                 raise exceptions.TimeoutException(msg)
 
             _, ex_body = self.get_object(url, ex_body['id'])
@@ -237,7 +243,9 @@ class AuthProv(auth.KeystoneV2AuthProvider):
 class TestCase(test.BaseTestCase):
     @classmethod
     def resource_setup(cls):
-        """This method allows to initialize authentication before
+        """Client authentication.
+
+        This method allows to initialize authentication before
         each test case and define parameters of Mistral API Service.
         """
         super(TestCase, cls).resource_setup()
@@ -246,7 +254,10 @@ class TestCase(test.BaseTestCase):
             cls.mgr = mock.MagicMock()
             cls.mgr.auth_provider = AuthProv()
         else:
-            cls.mgr = clients.Manager()
+            cls.creds = creds.get_configured_credentials(
+                credential_type='user'
+            )
+            cls.mgr = clients.Manager(cls.creds)
 
         if cls._service == 'workflowv2':
             cls.client = MistralClientV2(
